// 

import { asObject, asString } from 'cleaners'





const asEdgeRatesResponse = asObject({
  exchangeRate: asString
})

function checkIfFiat(code) {
  if (code.indexOf('iso:') >= 0) return true
  return false
}

export function makeEdgeRatesPlugin(
  opts
) {
  const { io, log } = opts
  const { fetch } = io

  return {
    rateInfo: {
      pluginId: 'edgeRates',
      displayName: 'EdgeRates'
    },

    async fetchRates(pairsHint) {
      const pairs = []
      for (const pair of pairsHint) {
        // Skip if neither code is a fiat code
        if (!checkIfFiat(pair.fromCurrency) || !checkIfFiat(pair.toCurrency))
          continue

        const fiatFrom = pair.fromCurrency.split(':')
        const fiatTo = pair.toCurrency.split(':')
        try {
          const reply = await fetch(
            `https://rates1.edge.app/v1/exchangeRate?currency_pair=${fiatFrom[1]}_${fiatTo[1]}`
          )
          const jsonData = await reply.json()
          const rate = Number(asEdgeRatesResponse(jsonData).exchangeRate)
          pairs.push({
            fromCurrency: pair.fromCurrency,
            toCurrency: pair.toCurrency,
            rate
          })
        } catch (e) {
          log.warn(
            `Issue with EdgeRates rate data structure for ${pair.fromCurrency}/${pair.toCurrency} pair. Error: ${e}`
          )
        }
      }
      return pairs
    }
  }
}
