








/**
 * Ensures that a date is in the future by at least the given amount.
 */
export function ensureInFuture(
  date,
  marginSeconds = 30
) {
  if (date == null) return
  const target = Date.now() + marginSeconds * 1000
  return target < date.valueOf() ? date : new Date(target)
}

export function makeSwapPluginQuote(
  request,
  fromNativeAmount,
  toNativeAmount,
  tx,
  destinationAddress,
  pluginId,
  isEstimate = false,
  expirationDate,
  quoteId
) {
  const { fromWallet } = request

  const out = {
    fromNativeAmount,
    toNativeAmount,
    networkFee: {
      currencyCode: fromWallet.currencyInfo.currencyCode,
      nativeAmount:
        tx.parentNetworkFee != null ? tx.parentNetworkFee : tx.networkFee
    },
    destinationAddress,
    pluginId,
    expirationDate,
    quoteId,
    isEstimate,
    async approve() {
      const signedTransaction = await fromWallet.signTx(tx)
      const broadcastedTransaction = await fromWallet.broadcastTx(
        signedTransaction
      )
      await fromWallet.saveTx(signedTransaction)

      return {
        transaction: broadcastedTransaction,
        orderId: quoteId,
        destinationAddress
      }
    },

    async close() {}
  }
  return out
}
